@@ -54,10 +54,12 @@ def __json_encoder_proquo(obj):
 
 
 def __process_file(pro_quo, filename, source_file_content, target_file_content, quid_matches, output_folder_path,
-                   source_text_parallel_print, export_text, export_ref, output_type, csv_sep):
+                   source_text_parallel_print, parallel_print_first_page, parallel_print_last_page, export_text,
+                   export_ref, output_type, csv_sep):
 
     short_matches: List[MatchRef] = pro_quo.compare(source_file_content, target_file_content, quid_matches,
-                                                    source_text_parallel_print)
+                                                    source_text_parallel_print, parallel_print_first_page,
+                                                    parallel_print_last_page)
 
     if not export_text:
         for match in short_matches:
@@ -146,15 +148,16 @@ def __test_relation_bert(test_file_path, tokenizer_folder_path, model_folder_pat
 
 
 def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_file_path, rel_tokenizer_folder_path,
-                  rel_model_folder_path, quid_match_path, output_folder_path, parallel_print_files, export_text,
-                  export_ref, output_type, csv_sep, open_quote, close_quote):
+                  rel_model_folder_path, quid_match_path, output_folder_path, parallel_print_files,
+                  parallel_print_first_page, parallel_print_last_page, export_text, export_ref, output_type, csv_sep,
+                  open_quote, close_quote):
 
     reference_vectorizer = ReferenceVectorizer.from_vocab_file(ref_vocab_file_path, 25, True)
     reference_model_trainer = ReferenceModelTrainer(25, True, 32, 32, 0.2, 512, 10)
     reference_model = reference_model_trainer.get_model(reference_vectorizer.max_id)
     reference_model.load_weights(ref_model_file_path)
 
-    relation_vectorizer = RelationVectorizerBert.from_saved(300, rel_tokenizer_folder_path, True)
+    relation_vectorizer = RelationVectorizerBert.from_saved(200, rel_tokenizer_folder_path, True)
     relation_model = transformers.TFBertForSequenceClassification.from_pretrained(rel_model_folder_path, num_labels=2)
 
     with open(source_file_path, 'r', encoding='utf-8') as source_file:
@@ -181,7 +184,8 @@ def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_
             source_text_parallel_print = True
 
         __process_file(pro_quo, filename, source_file_content, target_file_content, quid_matches, output_folder_path,
-                       source_text_parallel_print, export_text, export_ref, output_type, csv_sep)
+                       source_text_parallel_print, parallel_print_first_page, parallel_print_last_page, export_text,
+                       export_ref, output_type, csv_sep)
     elif isdir(target_path):
         for fileOrFolder in listdir(target_path):
             target_file_path = join(target_path, fileOrFolder)
@@ -205,8 +209,8 @@ def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_
                     source_text_parallel_print = True
 
                 __process_file(pro_quo, filename, source_file_content, target_file_content, quid_matches,
-                               output_folder_path, source_text_parallel_print, export_text, export_ref, output_type,
-                               csv_sep)
+                               output_folder_path, source_text_parallel_print, parallel_print_first_page,
+                               parallel_print_last_page, export_text, export_ref, output_type, csv_sep)
 
 
 def main(argv=None):
@@ -339,7 +343,12 @@ def main(argv=None):
     parser_compare.add_argument('--no-create-dated-subfolder', dest='create_dated_subfolder',
                                 action='store_false',
                                 help='Do not create a subfolder named with the current date to store the results')
-    parser_compare.add_argument('--parallel-print-files', dest='parallel_print_files', nargs='*', default=[])
+    parser_compare.add_argument('--parallel-print-files', dest='parallel_print_files', nargs='*', default=[],
+                                help='TBD')
+    parser_compare.add_argument('--parallel-print-first-page', dest='parallel_print_first_page', default=0,
+                                help='TBD')
+    parser_compare.add_argument('--parallel-print-last-page', dest='parallel_print_last_page', default=0,
+                                help='TBD')
     parser_compare.add_argument('--text', dest="export_text", default=True, action='store_true',
                                 help="Include matched text in the returned data structure")
     parser_compare.add_argument('--no-text', dest='export_text', action='store_false',
@@ -426,6 +435,8 @@ def main(argv=None):
         output_folder_path = args.output_folder_path
         create_dated_subfolder = args.create_dated_subfolder
         parallel_print_files = args.parallel_print_files
+        parallel_print_first_page = args.parallel_print_first_page
+        parallel_print_last_page = args.parallel_print_last_page
         export_text = args.export_text
         export_ref = args.export_ref
         output_type = args.output_type
@@ -441,7 +452,8 @@ def main(argv=None):
 
         __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_file_path,
                       rel_tokenizer_folder_path, rel_model_folder_path, quid_match_path, output_folder_path,
-                      parallel_print_files, export_text, export_ref, output_type, csv_sep, open_quote, close_quote)
+                      parallel_print_files, parallel_print_first_page, parallel_print_last_page, export_text,
+                      export_ref, output_type, csv_sep, open_quote, close_quote)
 
 
 if __name__ == '__main__':
