@@ -102,7 +102,7 @@ def __test(test_file_path, tokenizer_folder_path, model_folder_path):
 
 
 def __run_compare(source_file_path, target_path, tokenizer_folder_path, model_folder_path, quid_match_path,
-                  output_folder_path, export_text, output_type, csv_sep):
+                  output_folder_path, export_text, output_type, csv_sep, open_quote, close_quote):
 
     link_vectorizer = LinkingVectorizer.from_saved(512, tokenizer_folder_path, True)
     link_model = transformers.TFBertForSequenceClassification.from_pretrained(model_folder_path, num_labels=2)
@@ -110,7 +110,7 @@ def __run_compare(source_file_path, target_path, tokenizer_folder_path, model_fo
     with open(source_file_path, 'r', encoding='utf-8') as source_file:
         source_file_content = source_file.read().lower()
 
-    pro_quo_lm = ProQuoLm(link_model, link_vectorizer)
+    pro_quo_lm = ProQuoLm(link_model, link_vectorizer, open_quote, close_quote)
 
     if isfile(target_path) and target_path.endswith('.txt'):
         with open(target_path, 'r', encoding='utf-8') as target_file:
@@ -215,6 +215,12 @@ def main(argv=None):
                                 help="The output type")
     parser_compare.add_argument('--csv-sep', dest='csv_sep', type=str, help="output separator for csv (default: '\\t')",
                                 default='\t')
+    parser_compare.add_argument('--open-quote', dest='open_quote', type=str, help="The quotation open character"
+                                                                                  " (default: '\"')",
+                                default='"')
+    parser_compare.add_argument('--close-quote', dest='close_quote', type=str, help="The quotation close character"
+                                                                                    " (default: '\"')",
+                                default='"')
 
     args = argument_parser.parse_args(argv)
 
@@ -253,6 +259,8 @@ def main(argv=None):
         export_text = args.export_text
         output_type = args.output_type
         csv_sep = bytes(args.csv_sep, "utf-8").decode("unicode_escape")
+        open_quote = args.open_quote
+        close_quote = args.close_quote
 
         if create_dated_subfolder:
             now = datetime.now()
@@ -261,7 +269,7 @@ def main(argv=None):
             Path(output_folder_path).mkdir(parents=True, exist_ok=True)
 
         __run_compare(source_file_path, target_path, tokenizer_folder_path, model_folder_path, quid_match_path,
-                      output_folder_path, export_text, output_type, csv_sep)
+                      output_folder_path, export_text, output_type, csv_sep, open_quote, close_quote)
 
 
 if __name__ == '__main__':
