@@ -13,6 +13,11 @@ from proquo.core.Reference import Reference
 import numpy as np
 from sklearn.cluster import SpectralClustering
 import tensorflow as tf
+from transformers.modeling_tf_utils import TFPreTrainedModel
+from tensorflow.keras.models import Model
+
+from proquo.model.reference.ReferenceVectorizer import ReferenceVectorizer
+from proquo.model.relation.RelationVectorizerBert import RelationVectorizerBert
 
 
 # noinspection PyMethodMayBeStatic
@@ -29,17 +34,17 @@ class ProQuo:
     REL_MAX_WORDS: int = 120
     # Linking
     WITHOUT_REF_SEARCH_RADIUS = 500
-    SOURCE_PARALLEL_LAST_PAGE = 63
     # Fuzzy matching
     SCORE_CUTOFF: int = 0.85
 
-    def __init__(self, reference_model, reference_vectorizer, relation_model, relation_vectorizer, open_quote,
-                 close_quote):
+    def __init__(self, reference_model: Model, reference_vectorizer: ReferenceVectorizer,
+                 relation_model: TFPreTrainedModel, relation_vectorizer: RelationVectorizerBert, open_quote: str,
+                 close_quote: str):
         """
-        :param reference_model:
-        :param reference_vectorizer:
-        :param relation_model:
-        :param relation_vectorizer:
+        :param reference_model: The model for reference classification
+        :param reference_vectorizer: The vectorizer to use to tokenize and encode inputs for the reference model
+        :param relation_model: The model for relation classification
+        :param relation_vectorizer: The vectorizer to use to tokenize and encode inputs for the relation model
         """
         self.reference_model = reference_model
         self.reference_vectorizer = reference_vectorizer
@@ -47,19 +52,24 @@ class ProQuo:
         self.relation_vectorizer = relation_vectorizer
         self.source_cache = {}
         self.hashes = None
-        self.source_text_parallel_print = False
+        self.parallel_print = False
+        self.parallel_print_first_page = 0
+        self.parallel_print_last_page = 0
         self.open_quote = open_quote
         self.close_quote = close_quote
 
     def compare(self, source_text: str, target_text: str, quid_matches: List[Match],
-                source_text_parallel_print=False) -> List[MatchRef]:
+                parallel_print=False, parallel_print_first_page=0, parallel_print_last_page=0)\
+            -> List[MatchRef]:
         """
         Compare the two input texts and return a list of matching sequences.
         :param source_text: A source text
         :param target_text: A target text
         :param quid_matches: A list of matches from Quid to be used as candidates for short matches
-        :param source_text_parallel_print:
-        :return: A list of found matches
+        :param parallel_print: True if the source text is a parallel print edition
+        :param parallel_print_last_page: The first page on which the parallel print edition starts
+        :param parallel_print_first_page: The last page on which the parallel print edition appears
+        :return: A list of matches
         """
 
         source_text = Helper.clean_text(source_text)
@@ -72,15 +82,19 @@ class ProQuo:
             self.hashes = Helper.init_lsh_hashes(source_text)
             self.source_cache[source_text_hash] = self.hashes
 
-        self.source_text_parallel_print = source_text_parallel_print
+        self.parallel_print = parallel_print
+        self.parallel_print_first_page = parallel_print_first_page
+        self.parallel_print_last_page = parallel_print_last_page
 
+        # sort Quid matches into two lists: long and short matches
         long_matches = self.__filter_long_matches(source_text, quid_matches)
         short_matches = Helper.filter_short_matches(source_text, target_text, quid_matches, self.LONG_MIN_LENGTH)
 
         matches_with_reference: List[MatchRef] = self.__add_reference_to_match(target_text, long_matches)
         matches_with_reference.sort(key=lambda x: x.target_span.end - x.target_span.start, reverse=True)
-        longest_matches: List[MatchRef] = []
 
+        # find the n longest matches with a reference
+        longest_matches: List[MatchRef] = []
         for match_with_reference in matches_with_reference:
             if match_with_reference.reference:
                 if re.search(r'\d', match_with_reference.reference.text):
@@ -89,19 +103,19 @@ class ProQuo:
             if len(longest_matches) >= self.REF_EXAMPLE_COUNT:
                 break
 
-        all_references: List[Reference] = self.__get_references(target_text, longest_matches)
+        prototypical_reference = self.__get_prototypical_reference(longest_matches)
+        all_references: List[Reference] = self.__get_references(target_text, prototypical_reference)
         all_quotes: List[Quote] = Helper.get_quotations(target_text, self.LONG_MIN_LENGTH, self.open_quote,
                                                         self.close_quote)
         footnote_ranges = Helper.get_footnote_ranges(target_text)
 
+        # we are only interested in quotes and references in the running text
         main_references = []
-
         for ref in all_references:
             if not Helper.is_in_footnote(ref.start, ref.end, footnote_ranges):
                 main_references.append(ref)
 
         main_quotes = []
-
         for q in all_quotes:
             if not Helper.is_in_footnote(q.start, q.end, footnote_ranges):
                 main_quotes.append(q)
@@ -199,7 +213,7 @@ class ProQuo:
         preds = list(self.reference_model.predict([test_data_x_1, test_data_x_2], verbose=1).ravel())
         return preds
 
-    def __predict_rel(self, quote_ref_combos):
+    def __predict_rel(self, quote_ref_combos: List[QuoteRef]) -> List[float]:
         if len(quote_ref_combos) == 0:
             return []
 
@@ -219,7 +233,7 @@ class ProQuo:
 
         return preds
 
-    def __filter_long_matches(self, input_text: str, matches):
+    def __filter_long_matches(self, input_text: str, matches: List[Match]) -> List[Match]:
         result = []
         for match in matches:
             text = input_text[match.source_span.start:match.source_span.end]
@@ -230,7 +244,14 @@ class ProQuo:
 
         return result
 
-    def __add_reference_to_match(self, input_text, matches) -> List[MatchRef]:
+    def __add_reference_to_match(self, input_text: str, matches: List[Match]) -> List[MatchRef]:
+        """
+        Goes through all matches and tries to find a reference in parentheses after the match within a certain distance.
+        :param input_text: A text
+        :param matches: A list of matches
+        :return: A list of matches with references
+        """
+
         result: List[MatchRef] = []
 
         for match in matches:
@@ -252,16 +273,23 @@ class ProQuo:
 
         return result
 
-    def __get_references(self, input_text: str, match_ref_examples: List[MatchRef]) -> List[Reference]:
+    def __get_prototypical_reference(self, match_ref_examples: List[MatchRef]) -> str:
+        """
+        From the up to five examples, the best example is identified.
+        :param match_ref_examples: A list of matches with references
+        :return: A prototypical reference string.
+        """
         num_examples = len(match_ref_examples)
 
         if num_examples == 0:
-            return []
+            return ''
 
         match_ref_ex_with_id = []
 
+        # as a default, we just use the first example
         index = 0
 
+        # To identify the best example, we need more than two examples
         if num_examples > 2:
             for pos, mre in enumerate(match_ref_examples):
                 match_ref_ex_with_id.append((pos, mre))
@@ -278,7 +306,7 @@ class ProQuo:
                 combinations.append((perm[0], perm[1]))
 
             if len(sentences_1) == 0 or len(sentences_2) == 0:
-                return []
+                return ''
 
             preds = self.__predict_ref(sentences_1, sentences_2)
             num_examples = len(match_ref_examples)
@@ -304,6 +332,18 @@ class ProQuo:
                 index = labels.index(1)
 
         best_bibl = match_ref_examples[index]
+        return best_bibl.reference.text
+
+    def __get_references(self, input_text: str, prototypical_reference: str) -> List[Reference]:
+        """
+        All text in parentheses is compared to the prototypical example to identify all references in the text.
+        :param input_text: A text
+        :param prototypical_reference: A prototypical reference string
+        :return: A list of all references in the text which are similar to the given examples.
+        """
+
+        if not prototypical_reference:
+            return []
 
         bracket_positions = {}
         stack = []
@@ -327,7 +367,7 @@ class ProQuo:
             page_nr = self.__extract_page_from_reference(ref_text)
             possible_references.append(Reference(start + 1, end, input_text[start + 1:end], page_nr))
 
-        possible_permutations = itertools.product([x.text for x in possible_references], [best_bibl.reference.text])
+        possible_permutations = itertools.product([x.text for x in possible_references], [prototypical_reference])
 
         poss_bibl_sentences_1 = []
         poss_bibl_sentences_2 = []
@@ -346,7 +386,15 @@ class ProQuo:
 
         return references
 
-    def __replace_other_ref(self, input_text, references, start, end):
+    def __replace_other_ref(self, input_text: str, references: List[Reference], start: int, end: int) -> str:
+        """
+        Replace reference strings in the given text with a tag in the range between start and end.
+        :param input_text: A text
+        :param references: A list of references
+        :param start: A start character position
+        :param end: An end character position
+        :return: The modified input text
+        """
         other_refs = []
 
         for ref in references:
@@ -354,7 +402,6 @@ class ProQuo:
                 other_refs.append(ref)
 
         text = input_text[start:end]
-
         other_refs.sort(key=lambda x: x.start, reverse=True)
 
         for ref in other_refs:
@@ -364,7 +411,18 @@ class ProQuo:
 
         return text
 
-    def __get_combinations(self, input_text, references, quotes, footnote_ranges: Optional[List], offset=0):
+    def __get_combinations(self, input_text: str, references: List[Reference], quotes: List[Quote],
+                           footnote_ranges: Optional[List[Tuple[int, int]]]) -> List[QuoteRef]:
+        """
+        Generate all combinations of quotes and reference within a certain distance, excluding footnotes.
+        :param input_text: The input text
+        :param references: A list of references
+        :param quotes: A list of quotes
+        :param footnote_ranges: A list of footnote ranges
+        :return: A list of combinations of quotes and references. Additionally, a text is created with the quote tagged
+        and the reference replaced with a tag.
+        """
+
         my_permutations = itertools.product(quotes, references)
 
         result: List[QuoteRef] = []
@@ -374,10 +432,10 @@ class ProQuo:
             reference = perm[1]
 
             if quote.start < reference.end:
-                start = quote.start - offset
-                end = reference.end - offset
-                between_start = quote.end - offset
-                between_end = reference.start - offset
+                start = quote.start
+                end = reference.end
+                between_start = quote.end
+                between_end = reference.start
                 words_in_fn_count = 0
 
                 if footnote_ranges:
@@ -390,10 +448,10 @@ class ProQuo:
                 text_replaced = self.__replace_other_ref(input_text, references, quote.end, reference.start)
                 text = f' <Q> {input_text[quote.start:quote.end]} </Q> {text_replaced[1:]} <REF> '
             else:
-                start = reference.start - offset
-                end = quote.end - offset
-                between_start = reference.end - offset
-                between_end = quote.start - offset
+                start = reference.start
+                end = quote.end
+                between_start = reference.end
+                between_end = quote.start
                 words_in_fn_count = 0
 
                 if footnote_ranges:
@@ -460,7 +518,7 @@ class ProQuo:
 
         return result
 
-    def __get_short_quotes(self, quote_ref_combos) -> List[QuoteRef]:
+    def __get_short_quotes(self, quote_ref_combos: List[QuoteRef]) -> List[QuoteRef]:
         preds = self.__predict_rel(quote_ref_combos)
         result = []
 
@@ -471,7 +529,15 @@ class ProQuo:
 
         return result
 
-    def __filter_quotes_not_in_source(self, quotes: List[Quote], start, end) -> List[Quote]:
+    def __filter_quotes_not_in_source(self, quotes: List[Quote], start: int, end: int) -> List[Quote]:
+        """
+        Create a list of quotes from the given list of quotes which only contains quotes which are longer than a single
+        word or could be found the source text by fuzzing matching.
+        :param quotes: A list of quotes
+        :param start: A start character position
+        :param end: A end character position
+        :return: A list of the remaining quotes
+        """
         result: List[Quote] = []
 
         for quote in quotes:
@@ -487,7 +553,12 @@ class ProQuo:
 
         return result
 
-    def __filter_duplicates(self, short_quotes):
+    def __filter_duplicates(self, short_quotes: List[QuoteRef]) -> List[QuoteRef]:
+        """
+        Create a list only containing non overlapping quotes.
+        :param short_quotes: The input list of quotes
+        :return: The resulting list of quotes
+        """
         result = []
 
         for sq in short_quotes:
@@ -509,21 +580,35 @@ class ProQuo:
 
         return result
 
-    def __count_odd_numbers_in_range(self, start, end):
+    def __calc_page_offset_parallel_print(self, start: int, end: int) -> int:
+        """
+        Only needed for parallel print editions.
+        :param start: A start page
+        :param end: An end page
+        :return: The number of pages to subtract
+        """
         if end > start:
-            if end <= self.SOURCE_PARALLEL_LAST_PAGE:
-                return (end - start) // 2
-            elif start < self.SOURCE_PARALLEL_LAST_PAGE:
-                return self.__count_odd_numbers_in_range(start, self.SOURCE_PARALLEL_LAST_PAGE)
+            if self.parallel_print_first_page < end <= self.parallel_print_last_page:
+                return (end - max(self.parallel_print_first_page, start)) // 2
+            elif self.parallel_print_first_page < start < self.parallel_print_last_page:
+                return (self.parallel_print_last_page - start) // 2
         else:
-            if start <= self.SOURCE_PARALLEL_LAST_PAGE:
-                return (start - end) // 2
-            elif end < self.SOURCE_PARALLEL_LAST_PAGE:
-                return self.__count_odd_numbers_in_range(end, self.SOURCE_PARALLEL_LAST_PAGE)
+            if self.parallel_print_first_page < start < self.parallel_print_last_page:
+                return (start - max(self.parallel_print_first_page, end)) // 2
+            elif self.parallel_print_first_page < end < self.parallel_print_last_page:
+                return (self.parallel_print_last_page - end) // 2
 
         return 0
 
-    def __extract_page_from_reference(self, text: str, prev_match_with_ref: Optional[MatchRef] = None) -> int:
+    def __extract_page_from_reference(self, text: str) -> int:
+        """
+        A simple approach to extracting a page number from a reference string.
+        :param text: A reference string
+        :return: The extracted page number
+        """
+
+        page_nr = 0
+
         re_match = re.search(r'S\. ?(\d+)', text)
         if re_match:
             page_nr = int(re_match.group(1))
@@ -539,19 +624,29 @@ class ProQuo:
             page_nr = int(re_match.group(1))
             return page_nr
 
-        page_nr = self.__resolve_non_number_page(text, prev_match_with_ref)
+        # page_nr = self.__resolve_non_number_page(text, prev_match_with_ref)
         return page_nr
 
-    def __resolve_non_number_page(self, text: str, prev_match_with_ref: MatchRef) -> int:
-        if not prev_match_with_ref:
-            return 0
+    # def __resolve_non_number_page(self, text: str, prev_match_with_ref: MatchRef) -> int:
+    #     if not prev_match_with_ref:
+    #         return 0
+    #
+    #     if text.startswith('ebd'):
+    #         return prev_match_with_ref.reference.page
+    #
+    #     return 0
 
-        if text.startswith('ebd'):
-            return prev_match_with_ref.reference.page
+    def __count_words_in_footnotes(self, input_text: str, footnote_ranges: List[Tuple[int, int]], start: int, end: int)\
+            -> int:
+        """
+        Count the number of words in footnotes in the given range.
+        :param input_text: The input text
+        :param footnote_ranges: The list of footnote ranges
+        :param start: A start character position
+        :param end: An end character position
+        :return: The number of words in footnotes
+        """
 
-        return 0
-
-    def __count_words_in_footnotes(self, input_text, footnote_ranges, start, end):
         count = 0
 
         for fr in footnote_ranges:
@@ -561,7 +656,7 @@ class ProQuo:
 
         return count
 
-    def __get_page_size(self, matches_with_reference) -> Tuple[int, int, int]:
+    def __get_page_size(self, matches_with_reference: List[MatchRef]) -> Tuple[int, int, int]:
         mwr_sorted = sorted(matches_with_reference, key=lambda x: x.source_span.start)
 
         start = -1
@@ -587,8 +682,8 @@ class ProQuo:
 
         page_size = 0
 
-        if self.source_text_parallel_print:
-            page_diff = last_page - first_page - self.__count_odd_numbers_in_range(first_page, last_page)
+        if self.parallel_print:
+            page_diff = last_page - first_page - self.__calc_page_offset_parallel_print(first_page, last_page)
         else:
             page_diff = last_page - first_page
 
@@ -597,7 +692,8 @@ class ProQuo:
 
         return page_size, start, first_page
 
-    def __get_quote_range_with_ref(self, sq, page_size, first_page_known_pos, first_page_nr, source_text_len):
+    def __get_quote_range_with_ref(self, sq: QuoteRef, page_size: int, first_page_known_pos: int, first_page_nr: int,
+                                   source_text_len: int):
         start = 0
         end = source_text_len
         page_known_pos = -1
@@ -605,9 +701,9 @@ class ProQuo:
         if sq.reference.page > 0:
             if page_size > 0 and first_page_nr > 0:
 
-                if self.source_text_parallel_print:
+                if self.parallel_print:
                     page_diff = sq.reference.page - first_page_nr - \
-                                self.__count_odd_numbers_in_range(first_page_nr, sq.reference.page)
+                                self.__calc_page_offset_parallel_print(first_page_nr, sq.reference.page)
                 else:
                     page_diff = sq.reference.page - first_page_nr
 
@@ -622,7 +718,8 @@ class ProQuo:
 
         return start, end, page_known_pos
 
-    def __get_quote_range_without_ref(self, sq, matches_with_reference, source_text_len, page_size):
+    def __get_quote_range_without_ref(self, sq: QuoteRef, matches_with_reference: List[MatchRef], source_text_len: int,
+                                      page_size: int) -> Tuple[int, int, int]:
         mwr_best = None
         best_dist = sys.maxsize
 
@@ -656,7 +753,8 @@ class ProQuo:
 
         return start, end, page_known_pos
 
-    def __search_single_word(self, quote, source_text, search_space_start, search_space_end, page_known_pos):
+    def __search_single_word(self, quote: Quote, source_text: str, search_space_start: int, search_space_end: int,
+                             page_known_pos: int) -> Tuple[int, int]:
         source_search_space = source_text[search_space_start:search_space_end]
         re_matches = Helper.strict_match(quote.text, source_search_space)
         strict_matches_count = len(re_matches)
@@ -707,7 +805,8 @@ class ProQuo:
 
         return -1, -1
 
-    def __search_multi_word(self, short_matches, sq, range_start, range_end, source_text):
+    def __search_multi_word(self, short_matches: List[Match], sq: QuoteRef, range_start: int, range_end: int,
+                            source_text: str) -> Tuple[int, int]:
         search_space = source_text[range_start:range_end]
         re_matches = Helper.strict_match(sq.quote.text, search_space)
 
